------------------------------------------------------------------------------
--
--  generic package Mandelbrot.Parallel (spec)
--
--  A parallel implementation for calculating a map of the Mandelbrot set.
--
------------------------------------------------------------------------------
--  Update information:
--
--  2001.07.30 (Jacob Sparre Andersen)
--    Written.
--
--  2001.08.07 (Jacob Sparre Andersen)
--    Renamed the Job_Server parameter from Line_Count to Height.
--
--  (Insert additional update information above this line.)
------------------------------------------------------------------------------
--  Standard packages:

------------------------------------------------------------------------------
--  Other packages:

------------------------------------------------------------------------------

generic

package Mandelbrot.Parallel is

   ---------------------------------------------------------------------------
   --  procedure Calculate:

   procedure Calculate (View : in     View_Description;
                        Map  :    out Map_Type);

   ---------------------------------------------------------------------------

private

   ---------------------------------------------------------------------------
   --  type Line_Type:

   type Line_Type is array (Natural range <>) of Pixel_Type;

   ---------------------------------------------------------------------------
   --  type Boolean_Array:

   type Boolean_Array is array (Natural range <>) of Boolean;

   ---------------------------------------------------------------------------
   --  protected type Job_Server:

   protected type Job_Server (Height : Positive) is
      procedure Get (Line       :    out Natural;
                     End_Of_Job :    out Boolean);
   private
      Next_Line : Positive := 1;
   end Job_Server;

   type Job_Server_Reference is access all Job_Server;

   ---------------------------------------------------------------------------
   --  protected type Result_Buffer:

   protected type Result_Buffer (Width  : Positive;
                                 Height : Positive) is
      procedure Put (Line : in     Natural;
                     Data : in     Line_Type);
      entry Complete (Map :    out Map_Type);
   private
      Remaining_Lines : Natural := Height;
      Filled_Lines    : Boolean_Array (1 .. Height) := (others => False);
      Pixmap          : Map_Type (1 .. Width, 1 .. Height);
   end Result_Buffer;

   type Result_Buffer_Reference is access all Result_Buffer;

   ---------------------------------------------------------------------------
   --  task type Calculator:

   task type Calculator (Job    : View_Description_Reference;
                         Server : Job_Server_Reference;
                         Result : Result_Buffer_Reference);

   type Calculator_Reference is access all Calculator;

   ---------------------------------------------------------------------------

end Mandelbrot.Parallel;
