------------------------------------------------------------------------------
--
--  generic package Mandelbrot.Serial (body)
--
--  A serial implementation for calculating a map of the Mandelbrot set.
--
------------------------------------------------------------------------------
--  Update information:
--
--  2001.07.30 (Jacob Sparre Andersen)
--    Written.
--
--  (Insert additional update information above this line.)
------------------------------------------------------------------------------
--  Standard packages:

------------------------------------------------------------------------------
--  Other packages:

------------------------------------------------------------------------------

package body Mandelbrot.Serial is

   ---------------------------------------------------------------------------
   --  procedure Calculate:

   procedure Calculate (View : in     View_Description;
                        Map  :    out Map_Type) is

      use Complex_Types;

      use type Mandelbrot.Pixel_Type;

      Re_Start : Real;
      Im_Start : Imaginary;
      Z_0, Z   : Complex;

   begin --  Calculate
      for Line in Map'Range (2) loop
         Im_Start := View.Im_Min + (View.Im_Max - View.Im_Min) *
                       Real (Line - Map'First (2)) / Real (View.Height);

         for Column in Map'Range (1) loop
            Re_Start := View.Re_Min + (View.Re_Max - View.Re_Min) *
                          Real (Column - Map'First (1)) / Real (View.Width);

            Z_0 := Re_Start + Im_Start;
            Z   := Z_0;

            Map (Column, Line) := Pixel_Type (View.Iteration_Limit) + 1;

         Iterate:
            for Iteration in 0 .. View.Iteration_Limit loop
               if Modulus (Z) > 2.0 then
                  Map (Column, Line) := Pixel_Type (Iteration);
                  exit Iterate;
               end if;

               Z := Z_0 + Z ** 2; --  Tjek den lige!
            end loop Iterate;
         end loop;
      end loop;
   end Calculate;

   ---------------------------------------------------------------------------

end Mandelbrot.Serial;
