------------------------------------------------------------------------------
--
--  generic package Mandelbrot (spec)
--
--  Base package for experiments with serial, parallel and distributed
--  implementations for mapping the Mandelbrot set.
--
------------------------------------------------------------------------------
--  Update information:
--
--  2001.07.30 (Jacob Sparre Andersen)
--    Written.
--
--  (Insert additional update information above this line.)
------------------------------------------------------------------------------
--  Standard packages:

with Ada.Numerics.Generic_Complex_Types;

------------------------------------------------------------------------------
--  Other packages:

with PGM;

------------------------------------------------------------------------------

generic

   type Real is digits <>;

package Mandelbrot is

   ---------------------------------------------------------------------------
   --  subtype Pixel_Type:

   subtype Pixel_Type is PGM.Grey_16_Bit;

   ---------------------------------------------------------------------------
   --  subtype Map_Type:

   subtype Map_Type is PGM.Pixmap_16_Bit;

   ---------------------------------------------------------------------------
   --  procedure Save:

   procedure Save (Name : in     String;
                   Map  : in     Map_Type) renames PGM.Save;

   ---------------------------------------------------------------------------
   --  package Complex_Types:

   package Complex_Types is new Ada.Numerics.Generic_Complex_Types (Real);

   ---------------------------------------------------------------------------
   --  type Iteration_Limits:

   type Iteration_Limits is
     new Positive range 1 .. Positive (Pixel_Type'Last) - 1;

   ---------------------------------------------------------------------------
   --  type View_Description:

   type View_Description is
      record
         Iteration_Limit : Iteration_Limits;
         Width, Height   : Positive;
         Re_Min, Re_Max  : Real;
         Im_Min, Im_Max  : Complex_Types.Imaginary;
      end record;

   type View_Description_Reference is access constant View_Description;

   ---------------------------------------------------------------------------

end Mandelbrot;
